/**
 *  
 *  BibSonomy-Rest-Client - The REST-client.
 *   
 *  Copyright (C) 2006 - 2010 Knowledge & Data Engineering Group, 
 *                            University of Kassel, Germany
 *                            http://www.kde.cs.uni-kassel.de/
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

package org.bibsonomy.rest.client.queries.put;

import java.io.StringWriter;

import org.bibsonomy.model.BibTex;
import org.bibsonomy.model.Bookmark;
import org.bibsonomy.model.Post;
import org.bibsonomy.model.Resource;
import org.bibsonomy.model.Tag;
import org.bibsonomy.rest.client.AbstractQuery;
import org.bibsonomy.rest.client.exception.ErrorPerformingRequestException;
import org.bibsonomy.rest.enums.HttpMethod;
import org.bibsonomy.rest.exceptions.BadRequestOrResponseException;
import org.bibsonomy.rest.renderer.RendererFactory;

/**
 * Use this Class to change details of an existing post - change tags, for
 * example.
 * 
 * @author Manuel Bork <manuel.bork@uni-kassel.de>
 * @version $Id: ChangePostQuery.java,v 1.12 2010-04-27 10:41:38 bibsonomy Exp $
 */
public final class ChangePostQuery extends AbstractQuery<String> {
	private final Post<? extends Resource> post;
	private final String username;
	private final String resourceHash;

	/**
	 * Changes details of an existing post.
	 * 
	 * @param username
	 *            the username under which the post is to be created
	 * @param resourceHash
	 *            hash of the resource to change
	 * @param post
	 *            the new value for the post
	 * @throws IllegalArgumentException
	 *             if
	 *             <ul>
	 *             <li>username or resourcehash are not specified</li>
	 *             <li>no resource is connected with the post</li>
	 *             <li>the resource is a bookmark: if no url is specified</li>
	 *             <li>the resource is a bibtex: if no title is specified</li>
	 *             <li>no tags are specified or the tags have no names</li>
	 *             </ul>
	 */
	public ChangePostQuery(final String username, final String resourceHash, final Post<? extends Resource> post) throws IllegalArgumentException {
		if (username == null || username.length() == 0) throw new IllegalArgumentException("no username given");
		if (resourceHash == null || resourceHash.length() == 0) throw new IllegalArgumentException("no resourceHash given");
		if (post == null) throw new IllegalArgumentException("no post specified");
		if (post.getResource() == null) throw new IllegalArgumentException("no resource specified");

		if (post.getResource() instanceof Bookmark) {
			final Bookmark bookmark = (Bookmark) post.getResource();
			if (bookmark.getUrl() == null || bookmark.getUrl().length() == 0) throw new IllegalArgumentException("no url specified in bookmark");
		}

		if (post.getResource() instanceof BibTex) {
			final BibTex bibtex = (BibTex) post.getResource();
// INTRAhash is enough information when changing a post			
//			if (bibtex.getInterHash() == null || bibtex.getInterHash().length() == 0) {
//				throw new IllegalArgumentException("found an bibtex without interhash assigned.");
//			}
			if (bibtex.getIntraHash() == null || bibtex.getIntraHash().length() == 0) {
				throw new IllegalArgumentException("found an bibtex without intrahash assigned.");
			}
		}

		if (post.getTags() == null || post.getTags().size() == 0) throw new IllegalArgumentException("no tags specified");
		for (final Tag tag : post.getTags()) {
			if (tag.getName().length() == 0) throw new IllegalArgumentException("missing tagname");
		}

		this.username = username;
		this.resourceHash = resourceHash;
		this.post = post;
	}

	@Override
	protected String doExecute() throws ErrorPerformingRequestException {
		final StringWriter sw = new StringWriter(100);
		RendererFactory.getRenderer(getRenderingFormat()).serializePost(sw, post, null);
		this.downloadedDocument = performRequest(HttpMethod.PUT, URL_USERS + "/" + this.username + "/" + URL_POSTS + "/" + resourceHash + "?format=" + getRenderingFormat().toString().toLowerCase(), sw.toString());
		return null;
	}
	
	@Override
	public String getResult() throws BadRequestOrResponseException, IllegalStateException {
		if (this.isSuccess())
			return RendererFactory.getRenderer(getRenderingFormat()).parseResourceHash(this.downloadedDocument); 
		return this.getError();
	}
}